"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sql_wasm_1 = __importDefault(require("sql-wasm"));
const path_1 = require("path");
const os_1 = __importDefault(require("os"));
const fs_extra_1 = __importStar(require("fs-extra"));
const tar_1 = __importDefault(require("tar"));
// the following doesn't have a type definition
const recursiveCopy = require("recursive-copy");
const util_1 = require("./util");
const docset_tools_types_1 = require("docset-tools-types");
function escape(value) {
    if (value) {
        return value.replace(/'/g, "''");
    }
}
function default_1(options, argv) {
    return __awaiter(this, void 0, void 0, function* () {
        const { indexPath, iconsPath, docsPath, isJavascriptEnabled, docsetPlatformFamily, fallbackUrl, plugins, } = options;
        let { docsetIdentifier, outputPath } = options;
        let entries = options.entries || {};
        if (options.indexPath) {
            entries.index = options.indexPath;
        }
        let indexFilePath = indexPath || entries.index;
        const keep = argv.keep;
        const outputFileName = argv.file;
        if (!docsetIdentifier) {
            docsetIdentifier = require(path_1.join(process.cwd(), "package.json")).name;
        }
        outputPath = outputPath ? docset_tools_types_1.normalizePath(outputPath) : process.cwd();
        const tempPath = path_1.join(os_1.default.tmpdir(), "docset-" + Math.floor(Math.random() * 1000));
        const docsetFileName = docsetIdentifier + ".docset";
        const outputBasePath = path_1.join(outputPath, outputFileName || docsetFileName.replace(/[\\\/]/g, "_"));
        const outputContentsPath = path_1.join(outputBasePath, "Contents");
        const outputResourcesPath = path_1.join(outputContentsPath, "Resources");
        const outputDocsPath = path_1.join(outputResourcesPath, "Documents");
        const docsetName = options.docsetName || docsetIdentifier;
        const dryRun = argv["dry-run"];
        yield util_1.rmdir(outputBasePath);
        yield util_1.rmdir(outputBasePath + ".tgz");
        const log = (...args) => {
            if (options.logToConsole) {
                console.log(...args);
            }
        };
        let tmpCount = 0;
        const plistAdditions = {};
        if (options.plist) {
            Object.entries(options.plist).forEach(([key, value]) => {
                if (!plistAdditions[key]) {
                    plistAdditions[key] = [];
                }
                plistAdditions[key].push(value);
            });
        }
        const createTmpFolder = () => __awaiter(this, void 0, void 0, function* () {
            tmpCount++;
            const path = path_1.join(tempPath, tmpCount.toString());
            yield fs_extra_1.ensureDir(path);
            return path;
        });
        const include = ({ path, rootDirName, appendToTop, appendToBottom, remove, }) => __awaiter(this, void 0, void 0, function* () {
            // copy the content to the output directory
            if (!dryRun) {
                if (appendToTop || appendToBottom) {
                    // move to a new tmp dir to isolate
                    const tmpPath = yield createTmpFolder();
                    yield recursiveCopy(path, tmpPath);
                    function doFileAdd(fullPath, content, top) {
                        if (content) {
                            const srcData = fs_extra_1.default.readFileSync(fullPath, {
                                encoding: "utf8",
                            });
                            const newData = top
                                ? content + "\n" + srcData
                                : srcData + "\n" + content;
                            fs_extra_1.default.writeFileSync(fullPath, newData, { encoding: "utf8" });
                        }
                    }
                    // append to top and/or bottom
                    const checkAdds = (subPath, entries, top) => {
                        if (!entries) {
                            return;
                        }
                        const fullPath = subPath ? path_1.join(tmpPath, subPath) : tmpPath;
                        const docsContents = fs_extra_1.readdirSync(fullPath);
                        for (let j = 0; j < docsContents.length; j++) {
                            const name = docsContents[j];
                            const childPath = path_1.join(fullPath, name);
                            const stats = fs_extra_1.statSync(childPath);
                            if (stats.isFile()) {
                                const relativeFilePath = (subPath
                                    ? path_1.join(subPath, name)
                                    : name).replace(/\\/g, "/");
                                const fileExt = path_1.extname(childPath);
                                doFileAdd(childPath, entries[relativeFilePath], top);
                                doFileAdd(childPath, entries[(subPath ? subPath + "/" : "") + "*" + fileExt], top);
                                doFileAdd(childPath, entries["**" + fileExt], top);
                            }
                            else if (stats.isDirectory()) {
                                const root = subPath ? path_1.join(subPath, name) : name;
                                checkAdds(root, entries, top);
                            }
                        }
                    };
                    checkAdds(undefined, appendToTop, true);
                    checkAdds(undefined, appendToBottom, false);
                    path = tmpPath;
                }
                const destPath = rootDirName
                    ? path_1.join(outputDocsPath, rootDirName)
                    : outputDocsPath;
                yield recursiveCopy(path, destPath);
                if (remove) {
                    yield util_1.rmdir(path);
                }
            }
        });
        const _options = Object.assign(Object.assign({}, options), { dryRun });
        try {
            for (let i = 0; i < plugins.length; i++) {
                const plugin = plugins[i];
                const data = yield plugin.plugin.execute({
                    cliArgs: argv,
                    createTmpFolder,
                    include,
                    pluginOptions: plugin.options || {},
                    mainOptions: _options,
                    workingDir: process.cwd(),
                });
                const _entries = data.entries;
                if (_entries.index) {
                    if (indexFilePath === undefined &&
                        (plugin.useAsIndex || plugins.length === 1)) {
                        indexFilePath = _entries.index;
                    }
                }
                entries = docset_tools_types_1.mergeEntries(entries, _entries);
                if (data.plist) {
                    Object.entries(data.plist).forEach(([key, value]) => {
                        if (!plistAdditions[key]) {
                            plistAdditions[key] = [];
                        }
                        plistAdditions[key].push(value);
                    });
                }
            }
            const indexPathParts = indexFilePath ? indexFilePath.split("/") : [];
            let indexFileName = indexPathParts.pop() || "index.html";
            delete entries.index;
            if (docsPath) {
                const fullDocsPath = docset_tools_types_1.normalizePath(docsPath);
                log("copying from " + fullDocsPath + " to " + outputDocsPath);
                yield recursiveCopy(fullDocsPath, outputDocsPath);
            }
            log("creating index");
            const wasmPath = require.resolve("sql-wasm/dist/sqlite3.wasm");
            const SQL = yield sql_wasm_1.default({ wasmUrl: wasmPath });
            const db = new SQL.Database();
            const commands = [
                `CREATE TABLE searchIndex(id INTEGER PRIMARY KEY, name TEXT, type TEXT, path TEXT);`,
                `CREATE UNIQUE INDEX anchor ON searchIndex (name, type, path);`,
            ];
            const fileRefs = [];
            Object.entries(entries).forEach(([type, entries]) => {
                function normalizePath(path) {
                    let _path = path;
                    _path = _path.replace(/[\\/]#/, "#").replace(/\\/g, "/");
                    if (_path.endsWith("/")) {
                        _path = _path + indexFileName;
                    }
                    if (_path.startsWith("#") && indexFilePath) {
                        _path = indexFilePath + _path;
                    }
                    if (_path.startsWith("/")) {
                        _path = _path.replace(/^\//, "");
                    }
                    _path = _path.replace(/\\/g, "/");
                    return _path;
                }
                Object.entries(entries).forEach(([name, path]) => {
                    const _path = normalizePath(path);
                    fileRefs.push({ type: type, path: _path, name });
                    commands.push(`INSERT OR IGNORE INTO searchIndex(name, type, path) VALUES ('${escape(name)}', '${escape(type)}', '${escape(_path)}');`);
                });
            });
            for (let i = 0; i < commands.length; i++) {
                yield db.exec(commands[i]);
            }
            // save to file
            const data = db.export();
            const buffer = Buffer.from(data);
            fs_extra_1.ensureDirSync(outputResourcesPath);
            yield fs_extra_1.default.writeFile(path_1.join(outputResourcesPath, "docSet.dsidx"), buffer);
            // create info.plist
            log("creating info.plist");
            const infoPlistData = `
      <?xml version="1.0" encoding="UTF-8"?>
      <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
      <plist version="1.0">
      <dict>
        <key>CFBundleIdentifier</key>
        <string>${docsetIdentifier}</string>
        <key>CFBundleName</key>
        <string>${docsetName}</string>${docsetPlatformFamily
                ? `
        <key>DocSetPlatformFamily</key>
        <string>${docsetPlatformFamily}</string>`
                : ""}
        <key>isDashDocset</key>
        <true/>${indexFilePath
                ? `
        <key>dashIndexFilePath</key>
        <string>${indexFilePath}</string>`
                : ""}
        <key>isJavaScriptEnabled</key><${isJavascriptEnabled ? "true" : "false"}/>${fallbackUrl
                ? `
        <key>DashDocSetFallbackURL</key>
        <string>${fallbackUrl}</string>`
                : ""}${Object.entries(plistAdditions).map(([key, entries]) => `
        <key>${key}</key>${entries.map((value) => `
        <string>${value
                .replace(/&/g, "&amp")
                .replace(/</g, "&lt")
                .replace(/>/g, "&gt")}</string>`)}
        `)}
      </dict>
      </plist>
`;
            yield fs_extra_1.default.writeFile(path_1.join(outputContentsPath, "info.plist"), infoPlistData);
            // icons
            if (iconsPath) {
                log("copying icons...");
                const iconPath = path_1.join(iconsPath, "icon.png");
                const iconExists = fs_extra_1.existsSync(iconPath);
                if (iconExists) {
                    yield fs_extra_1.copy(iconPath, path_1.join(outputBasePath, "icon.png"));
                }
                else {
                    console.error(iconPath + " does not exist");
                }
                const icon2xPath = path_1.join(iconsPath, "icon@2x.png");
                const icon2xExists = fs_extra_1.existsSync(iconPath);
                if (icon2xExists) {
                    yield fs_extra_1.copy(icon2xPath, path_1.join(outputBasePath, "icon@2x.png"));
                }
                else {
                    console.error(icon2xPath + " does not exist");
                }
            }
            fileRefs.forEach(({ type, name, path }) => {
                if (path.startsWith("#")) {
                    path = indexFilePath ? indexFilePath + path : path;
                }
                // make sure the file is valid
                path = path_1.normalize(path_1.join(outputDocsPath, path));
                const pathToCheck = path.replace(/#.*/, "").replace(/\?.*/, "");
                path = "file://" + encodeURI(path.replace(/\\/g, "/"));
                if (type) {
                    console.log(type + " > " + name + "\n\t" + path);
                }
                else {
                    console.log("[" + name + "]\n\t" + path);
                }
                if (!fs_extra_1.default.existsSync(pathToCheck)) {
                    throw new Error(`${path} not found`);
                }
            });
            // tar it up
            yield tar_1.default.c({
                gzip: true,
                portable: true,
                file: outputBasePath + ".tgz",
                cwd: outputBasePath,
            }, ["./"]);
            if (!keep) {
                yield util_1.rmdir(outputBasePath);
            }
        }
        finally {
            try {
                yield util_1.rmdir(tempPath);
            }
            catch (e) {
                console.error(e);
            }
        }
    });
}
exports.default = default_1;
//# sourceMappingURL=engine.js.map